import { AppModel, AppListModel } from './module/index.ts';
import { app } from '@/app.ts';
import { setExpire } from './revoke.ts';
import { deleteFileByPrefix } from '../file/index.ts';

app
  .route({
    path: 'user-app',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const list = await AppModel.findAll({
      order: [['updatedAt', 'DESC']],
      where: {
        uid: tokenUser.id,
      },
      attributes: {
        exclude: ['data'],
      },
    });
    ctx.body = list;
    return ctx;
  })
  .addTo(app);

app
  .route({
    path: 'user-app',
    key: 'get',
    middleware: ['auth'],
    description: '获取用户应用,可以指定id或者key',
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const id = ctx.query.id;
    const { key } = ctx.query.data || {};
    if (!id && !key) {
      ctx.throw(500, 'id is required');
    }
    let am: AppModel;
    if (id) {
      am = await AppModel.findByPk(id);
      if (!am) {
        ctx.throw(500, 'app not found');
      }
    } else {
      am = await AppModel.findOne({ where: { key, uid: tokenUser.id } });
      if (!am) {
        ctx.throw(500, 'app not found');
      }
    }

    ctx.body = am;

    return ctx;
  })
  .addTo(app);

app
  .route({
    path: 'user-app',
    key: 'update',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;

    const { data, id, user, ...rest } = ctx.query.data;
    if (id) {
      const app = await AppModel.findByPk(id);
      if (app) {
        const newData = { ...app.data, ...data };
        if (app.user !== tokenUser.username) {
          rest.user = tokenUser.username;
          let files = newData?.files || [];
          if (files.length > 0) {
            files = files.map((item) => {
              const paths = item.path.split('/');
              return {
                ...item,
                path: newData.user + '/' + paths.slice(1).join('/'),
              };
            });
          }
          newData.files = files;
        }
        const newApp = await app.update({ data: newData, ...rest });
        ctx.body = newApp;
        if (app.status !== 'running') {
          setExpire(newApp.key, app.user);
        }
      } else {
        ctx.throw(500, 'app not found');
      }
      return;
    }
    if (!rest.key) {
      ctx.throw(500, 'key is required');
    }
    const findApp = await AppModel.findOne({ where: { key: rest.key, uid: tokenUser.id } });
    if (findApp) {
      ctx.throw(500, 'key already exists');
    }
    const app = await AppModel.create({
      data: { files: [] },
      ...rest,
      uid: tokenUser.id,
      user: tokenUser.username,
    });
    ctx.body = app;
    return ctx;
  })
  .addTo(app);

app
  .route({
    path: 'user-app',
    key: 'delete',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const id = ctx.query.id;
    const deleteFile = !!ctx.query.deleteFile; // 是否删除文件, 默认不删除
    if (!id) {
      ctx.throw(500, 'id is required');
    }
    const am = await AppModel.findByPk(id);
    if (!am) {
      ctx.throw(500, 'app not found');
    }
    if (am.uid !== tokenUser.id) {
      ctx.throw(500, 'app not found');
    }
    const list = await AppListModel.findAll({ where: { key: am.key, uid: tokenUser.id } });
    await am.destroy({ force: true });
    await Promise.all(list.map((item) => item.destroy({ force: true })));
    if (deleteFile) {
      const username = tokenUser.username;
      await deleteFileByPrefix(`${username}/${am.key}`);
    }
    ctx.body = 'success';
    return ctx;
  })
  .addTo(app);

app
  .route({
    path: 'user-app',
    key: 'test',
    description: '对user-app的数据进行测试, 获取版本的信息',
  })
  .define(async (ctx) => {
    const id = ctx.query.id;
    if (!id) {
      ctx.throw(500, 'id is required');
    }
    const am = await AppListModel.findByPk(id);
    if (!am) {
      ctx.throw(500, 'app not found');
    }
    const amJson = am.toJSON();
    ctx.body = {
      ...amJson,
      proxy: true,
    };
  })
  .addTo(app);
